// Michael J. Wilson
// Music 256a Fall 2010
// Homework #2

#include "UnitKarplusStrong.h"
#include "MidiKarplusStrongInstrument.h"
#include "Utility.h"
#include <cassert>

MidiKarplusStrongInstrument::~MidiKarplusStrongInstrument()
{
    for (std::vector<UnitKarplusStrong*>::iterator i = m_strings.begin(); i != m_strings.end(); ++i)
    {
        delete *i;
    }
}

MidiKarplusStrongInstrument::MidiKarplusStrongInstrument(unsigned int polyphony, unsigned int lowest_note, double base_attenuation)
    : m_polyphony(polyphony),
      m_lowest_note(lowest_note),
      m_base_attenuation(base_attenuation)
{
    for (unsigned int i=0; i<polyphony; ++i)
    {
        m_strings.push_back(new UnitKarplusStrong(Utility::midi_to_frequency(lowest_note), base_attenuation));
        m_free_string_indexes.push_back((size_t)i);
        m_string_note.push_back(lowest_note);
    }
}

void MidiKarplusStrongInstrument::play(unsigned int note, unsigned int velocity)
{
    assert (note >= m_lowest_note);
    assert (velocity <= 127);

    // The frequency of the note
    double frequency = Utility::midi_to_frequency(note);

    // The scaled gain
    double gain = (double)velocity / (double)127.0;

    // Take the first unused string.  We could also use this
    // datastructure for note-stealing polyphony, but I prefer
    // saturating polyphony since it is more obvious when the limit is
    // reached.
    if (!m_free_string_indexes.empty())
    {
        // Get the index of the first free string
        size_t i = m_free_string_indexes.front();
        // Remove the index from the list of free strings
        m_free_string_indexes.pop_front();
        // Put it in the list of used strings
        m_used_string_indexes.push_back(i);
        // Set parameters and excite the string
        m_strings[i]->set_frequency(frequency);
        m_strings[i]->set_attenuation(m_base_attenuation);
        m_strings[i]->play(gain);
        // Record which note this string is playing
        m_string_note[i] = note;
    }
}

void MidiKarplusStrongInstrument::release(unsigned int note)
{
    bool found_string = false;

    // This uses a few bookkeeping datastructures to make sure that
    // the string which was released last will be used first.
    for (std::list<size_t>::iterator i = m_used_string_indexes.begin(); !found_string && (i != m_used_string_indexes.end()); ++i)
    {
        if (m_string_note[*i] == note)
        {
            // Note we found the string so we don't continue iterating
            found_string = true;

            // Increase the attenuation of the string drastically
            m_strings[*i]->set_attenuation(m_base_attenuation / 2.0);

            // Mark the string as "free", that is, able to play another note
            m_free_string_indexes.push_back(*i);

            // Remove it from the list of used strings
            m_used_string_indexes.erase(i);
        }
    }
}

double MidiKarplusStrongInstrument::get_next_sample(double* input_sample)
{
    double result = 0.0;

    // Sum all the strings, then divide by the polyphony count.  Note
    // that we sum strings that are not playing as well.  This is by
    // design: strings that are released have their decay rate
    // increased and will eventually hit a threshold where they are
    // zeroed out.  Continuing to calculate released strings allows
    // them to decay naturally but still be repurposed when the
    // polyphony count is reached.
    for (size_t i = 0; i < m_strings.size(); ++i)
    {
        result += m_strings[i]->get_next_sample(input_sample);
    }

    result /= (double)m_polyphony;
    return result;
}
