// Michael J. Wilson
// Music 256a Fall 2010
// Homework #2

#ifndef __MIDIKARPLUSSTRONGINSTRUMENT_H__
#define __MIDIKARPLUSSTRONGINSTRUMENT_H__

#include "UnitGen.h"
#include <vector>
#include <list>

class UnitKarplusStrong;

/// An instrument that can play the Karplus Strong string model
class MidiKarplusStrongInstrument : public UnitGen
{
public:
    /// Destructor
    virtual ~MidiKarplusStrongInstrument();

    /// Constructor
    MidiKarplusStrongInstrument(unsigned int polyphony = 32, unsigned int lowest_note = 0, double base_attenuation = 0.99);

    /// Play the specified MIDI note at the specified velocity.  If
    /// there is not enough polyphony to support this request the note
    /// will not play.
    void play(unsigned int note, unsigned int velocity);

    /// Release the specified MIDI note.  If the same note has been
    /// polyphonically triggered multiple times, this will release one
    /// of the notes at random.
    void release(unsigned int note);

    /// Get the next sample to play
    double get_next_sample(double* input_sample);

protected:

    /// Strings in the instrument
    std::vector<UnitKarplusStrong*> m_strings;

    /// The MIDI note that is currently playing for the specified string
    std::vector<unsigned int> m_string_note;

    /// Indexes of free strings
    std::list<size_t> m_free_string_indexes;

    /// Indexes of used strings
    std::list<size_t> m_used_string_indexes;

    /// Number of notes that can be played simultaneously
    unsigned int m_polyphony;

    /// Lowest MIDI note that can be played
    unsigned int m_lowest_note;

    /// Base attenuation for the string model (0.9 and 0.99 are good
    /// choices).  Release attenuation is one-tenth this value.
    double m_base_attenuation;
};

#endif
