// Michael J. Wilson
// Music 256a Fall 2010
// Homework #2

#include "UnitKarplusStrong.h"
#include "Utility.h"
#include <cassert>
#include <cmath>

UnitKarplusStrong::~UnitKarplusStrong()
{
}

UnitKarplusStrong::UnitKarplusStrong(double frequency, double attenuation)
    : UnitDelayFeedback((size_t)((double)Utility::SAMPLE_RATE / frequency), attenuation),
      m_previous_output_sample(0)
{
}

void UnitKarplusStrong::set_attenuation(double attenuation)
{
    m_attenuation = attenuation;
}

void UnitKarplusStrong::set_frequency(double frequency)
{
    assert (m_buffer->size() >= (size_t)((double)Utility::SAMPLE_RATE / frequency));

    // Resize the loop, and reinitialize the pointers.  Doing it this
    // way avoids reallocating memory every time the frequency
    // changes.
    m_loop_size = (size_t)((double)Utility::SAMPLE_RATE / frequency);
    m_read_index = 0;
    m_write_index = (m_loop_size - 1);
}

void UnitKarplusStrong::play(double gain)
{
    assert (gain >= 0.0);

    // Fill with white noise
    for (std::vector<double>::iterator i = m_buffer->begin(); i != m_buffer->end(); ++i)
    {
        *i = Utility::get_utility()->rand(-gain, gain);
    }
}

double UnitKarplusStrong::get_next_sample(double* input_sample)
{
    double result = 0.0;

    if (m_buffer->size() > 0)
    {
        result = ((*m_buffer)[m_read_index] + m_previous_output_sample) / 2.0;
        m_previous_output_sample = result;
        if (std::abs(result) < Utility::EPSILON)
        {
            // Handle denormalization issues
            result = 0.0;
        }
        else
        {
            result *= m_attenuation;
        }
        (*m_buffer)[m_write_index] = result;

        m_read_index++;
        m_write_index++;

        m_read_index %= m_buffer->size();
        m_write_index %= m_buffer->size();
    }

    return (result);
}
