// Michael J. Wilson
// Music 256a Fall 2010
// Homework #2

#ifndef __UTILITY_H__
#define __UTILITY_H__

#include <string>

/// Waveform types
enum WaveformType
{
    SINE,
    SAW,
    PULSE,
    NOISE,
    IMPULSE
};

/// Singleton utility class, contains data that should be available to
/// the entire program
class Utility
{
public:
    /// Destructor
    ~Utility();

    /// Get the utility instance.  Lazily constructed.
    static Utility* get_utility();

    /// Ratio of circle's circumferance to its diameter
    static const double PI;

    /// Very small value for floating point comparisons
    static const double EPSILON;

    /// Sample rate
    static const unsigned int SAMPLE_RATE;

    /// Number of output channels
    static const unsigned int OUTPUT_CHANNELS;

    /// Name of the program
    static const std::string PROGRAM_NAME;

    /// Get a random number in the specified range.  No assurances on
    /// its distribution.
    double rand(double min, double max);

    /// Convert MIDI note to frequency
    static double midi_to_frequency(int note);

private:
    /// Private constructor to follow singleton pattern
    Utility();
};

#endif
